package org.apithefire.servlet.jetty;

import java.net.URL;

import javax.servlet.Servlet;

import org.apithefire.servlet.server.Context;
import org.apithefire.util.lang.Objects;
import org.apithefire.util.lang.Path;
import org.mortbay.jetty.handler.ContextHandlerCollection;
import org.mortbay.jetty.servlet.ServletHolder;
import org.mortbay.jetty.webapp.WebAppContext;

public class JettyContext extends Context {

	private static final String DUMMY_WAR = "dummy_war";
	private static final String DEFAULT_DESCRIPTOR = "DefaultDescriptor.xml";
	
	private final Provider<ContextHandlerCollection> contextsProvider;
	
	protected JettyContext(Provider<ContextHandlerCollection> contextsProvider,
			Path path) {
		super(path);
		this.contextsProvider = Objects.nonNull(contextsProvider);
	}
	
	@Override
	protected void configure() {
		super.configure();
		
		WebAppContext context = new WebAppContext();
		String contextPath = "/" + getPath().toString();
		URL dummyWar = getClass().getResource(DUMMY_WAR);
		URL defaultDescriptor = getClass().getResource(DEFAULT_DESCRIPTOR);
		Servlet servlet = getServlet();
		
		context.setContextPath(contextPath);
		context.setResourceBase(dummyWar.toString());
		context.setDefaultsDescriptor(defaultDescriptor.toString());

		if (servlet != null) {
			context.addServlet(new ServletHolder(servlet), "/*");
		}
		contextsProvider.get().addHandler(context);
	}

}
