package org.apithefire.servlet.jetty;

import org.apithefire.servlet.server.Server;
import org.apithefire.servlet.server.ServerException;
import org.apithefire.util.lang.Path;
import org.mortbay.jetty.handler.ContextHandlerCollection;

public class JettyServer extends Server {
	
	private final Provider<org.mortbay.jetty.Server> serverProvider =
			new Provider<org.mortbay.jetty.Server>() {
		@Override
		public org.mortbay.jetty.Server get() {
			return JettyServer.this.getServerImpl();
		}
	};
	
	private final Provider<ContextHandlerCollection> contextsProvider =
			new Provider<ContextHandlerCollection>() {
		@Override
		public ContextHandlerCollection get() {
			return (ContextHandlerCollection) JettyServer.this.getServerImpl()
					.getHandler();
		}
	};
	
	@Override
	protected JettyConnector createConnector(int port) {
		return new JettyConnector(serverProvider, port);
	}

	@Override
	protected JettyContext createContext(Path path) {
		return new JettyContext(contextsProvider, path);
	}

	@Override
	protected org.mortbay.jetty.Server createServerImpl() {
		org.mortbay.jetty.Server serverImpl = new org.mortbay.jetty.Server();
		
		serverImpl.setHandler(new ContextHandlerCollection());
		return serverImpl;
	}
	
	@Override
	protected org.mortbay.jetty.Server getServerImpl() {
		return (org.mortbay.jetty.Server) super.getServerImpl();
	}

	@Override
	public void start() {
		super.start();
		try {
			getServerImpl().start();
		} catch (RuntimeException e) {
			throw e;
		} catch (Exception e) {
			throw new ServerException(e);
		}
	}

	@Override
	public void join() {
		super.join();
		try {
			getServerImpl().join();
		} catch (RuntimeException e) {
			throw e;
		} catch (Exception e) {
			throw new ServerException(e);
		}
	}

	@Override
	public void stop() {
		super.stop();
		try {
			getServerImpl().stop();
		} catch (RuntimeException e) {
			throw e;
		} catch (Exception e) {
			throw new ServerException(e);
		}
	}

}
